@@ -19,11 +19,12 @@ import socket
 
 from keystoneauth1 import adapter
 from keystoneauth1 import exceptions as ksa_exc
+import OpenSSL
 from oslo_utils import importutils
 from oslo_utils import netutils
 import requests
 import six
-import warnings
+import six.moves.urllib.parse as urlparse
 
 try:
     import json
@@ -53,8 +54,26 @@ def encode_headers(headers):
     :returns: Dictionary with encoded headers'
               names and values
     """
-    return dict((encodeutils.safe_encode(h), encodeutils.safe_encode(v))
-                for h, v in six.iteritems(headers) if v is not None)
+    # NOTE(rosmaita): This function's rejection of any header name without a
+    # corresponding value is arguably justified by RFC 7230.  In any case, that
+    # behavior was already here and there is an existing unit test for it.
+
+    # Bug #1766235: According to RFC 8187, headers must be encoded as ASCII.
+    # So we first %-encode them to get them into range < 128 and then turn
+    # them into ASCII.
+    if six.PY2:
+        # incoming items may be unicode, so get them into something
+        # the py2 version of urllib can handle before percent encoding
+        encoded_dict = dict((urlparse.quote(encodeutils.safe_encode(h)),
+                             urlparse.quote(encodeutils.safe_encode(v)))
+                            for h, v in headers.items() if v is not None)
+    else:
+        encoded_dict = dict((urlparse.quote(h), urlparse.quote(v))
+                            for h, v in headers.items() if v is not None)
+
+    return dict((encodeutils.safe_encode(h, encoding='ascii'),
+                 encodeutils.safe_encode(v, encoding='ascii'))
+                for h, v in encoded_dict.items())
 
 
 class _BaseHTTPClient(object):
@@ -145,13 +164,6 @@ class HTTPClient(_BaseHTTPClient):
         self.timeout = float(kwargs.get('timeout', 600))
 
         if self.endpoint.startswith("https"):
-            compression = kwargs.get('ssl_compression', True)
-
-            if compression is False:
-                # Note: This is not seen by default. (python must be
-                # run with -Wd)
-                warnings.warn('The "ssl_compression" argument has been '
-                              'deprecated.', DeprecationWarning)
 
             if kwargs.get('insecure', False) is True:
                 self.session.verify = False
@@ -172,7 +184,7 @@ class HTTPClient(_BaseHTTPClient):
         headers = copy.deepcopy(headers)
         headers.update(self.session.headers)
 
-        for (key, value) in six.iteritems(headers):
+        for (key, value) in headers.items():
             header = '-H \'%s: %s\'' % utils.safe_header(key, value)
             curl.append(header)
 
@@ -218,7 +230,7 @@ class HTTPClient(_BaseHTTPClient):
         headers = copy.deepcopy(kwargs.pop('headers', {}))
 
         if self.identity_headers:
-            for k, v in six.iteritems(self.identity_headers):
+            for k, v in self.identity_headers.items():
                 headers.setdefault(k, v)
 
         data = self._set_common_request_kwargs(headers, kwargs)
@@ -267,6 +279,10 @@ class HTTPClient(_BaseHTTPClient):
             message = ("Error communicating with %(endpoint)s %(e)s" %
                        {'endpoint': endpoint, 'e': e})
             raise exc.CommunicationError(message=message)
+        except OpenSSL.SSL.Error as e:
+            message = ("SSL Error communicating with %(url)s: %(e)s" %
+                       {'url': conn_url, 'e': e})
+            raise exc.CommunicationError(message=message)
 
         # log request-id for each api call
         request_id = resp.headers.get('x-openstack-request-id')
